@@ -867,11 +867,11 @@ sub _do_dispatch {
     }
 
     # Close the handle down, so the client can go on their merry way
-    # XXX TODO FIXME: Doing this causes tests to explode with "message
-    # type 0x32 arrived from server while idle", which is a mesage
-    # from Postgres.  It's unclear why Pg cares that we closed STDOUT,
-    # but this yak is too big to shave right now.
-    #close STDOUT unless Jifty->web->request->is_subrequest;
+    unless (Jifty->web->request->is_subrequest) {
+        Jifty->handler->buffer->flush_output;
+        close(STDOUT);
+        $Jifty::SERVER->close_client_sockets if $Jifty::SERVER;
+    }
 
     # Cleanup
     $self->_handle_stage('CLEANUP');
@@ -1192,7 +1192,7 @@ Templates.  Specifically, returns a reference to the handler which can
 process the template.
 
 If PATH is a I<reference> to the path, it will update the path to
-append C</index.html> if the path in question doesn't exist, bit the
+append C</index.html> if the path in question doesn't exist, but the
 index does.
 
 =cut
