@@ -14,6 +14,7 @@ Jifty::Util - Things that don't fit anywhere else
 
 use Jifty ();
 use File::Spec ();
+use File::ShareDir ();
 use Cwd ();
 
 use vars qw/%ABSOLUTE_PATH $JIFTY_ROOT $SHARE_ROOT $APP_ROOT/;
@@ -60,10 +61,10 @@ sub canonicalize_path {
             next;
         }
         elsif( $_ ne '..')  {
-        push @newpath, $_ ;
-    } else {
-        pop @newpath;
-    }
+            push @newpath, $_ ;
+        } else {
+            pop @newpath;
+        }
 
     }
 
@@ -101,27 +102,29 @@ of Jifty and it's plugins.
 
 sub share_root {
     my $self = shift;
-
-    
-    Jifty::Util->require('File::ShareDir');
-    $SHARE_ROOT ||=  eval { File::Spec->rel2abs( File::ShareDir::module_dir('Jifty') )};
-    if (not $SHARE_ROOT or not -d $SHARE_ROOT) {
-        # XXX TODO: This is a bloody hack
-        # Module::Install::ShareDir and File::ShareDir don't play nicely
-        # together
+    unless (defined $SHARE_ROOT) {
+        # Try for the local version, first
         my @root = File::Spec->splitdir($self->jifty_root); # lib
         pop @root; # Jifty-version
         $SHARE_ROOT = File::Spec->catdir(@root,"share");
+        undef $SHARE_ROOT unless defined $SHARE_ROOT and -d $SHARE_ROOT and -d File::Spec->catdir($SHARE_ROOT,"web");
+
+        # If that doesn't pass inspection, try File::ShareDir::dist_dir
+        $SHARE_ROOT ||= eval { File::Spec->rel2abs( File::ShareDir::dist_dir('Jifty') )};
+        undef $SHARE_ROOT unless defined $SHARE_ROOT and -d $SHARE_ROOT and -d File::Spec->catdir($SHARE_ROOT,"web");
     }
+
+    die "Can't locate Jifty share root!" unless defined $SHARE_ROOT;
     return ($SHARE_ROOT);
 }
 
 =head2 app_root
 
-Returns the application's root path.  This is done by searching upward
-from the current directory, looking for a directory which contains a
-C<bin/jifty>.  Failing that, it searches upward from wherever the
-executable was found.
+Returns the application's root path.  This is done by returning
+$ENV{'JIFTY_APP_ROOT'} if it exists.  If not, Jifty tries searching
+upward from the current directory, looking for a directory which
+contains a C<bin/jifty>.  Failing that, it searches upward from
+wherever the executable was found.
 
 It C<die>s if it can only find C</usr> or C</usr/local> which fit
 these criteria.
@@ -130,8 +133,9 @@ these criteria.
 
 sub app_root {
     my $self = shift;
+    my %args = @_;
 
-
+    return $ENV{'JIFTY_APP_ROOT'} if ($ENV{'JIFTY_APP_ROOT'});
     return $APP_ROOT if ($APP_ROOT);
     
     my @roots;
@@ -147,10 +151,11 @@ sub app_root {
 
     Jifty::Util->require('ExtUtils::MM') if $^O =~ /(?:MSWin32|cygwin|os2)/;
     Jifty::Util->require('Config');
-    for (@roots) {
-        my @root = File::Spec->splitdir($_);
+    for my $root_path (@roots) {
+        my ($volume, $dirs) = File::Spec->splitpath($root_path, 'no_file');
+        my @root = File::Spec->splitdir($dirs);
         while (@root) {
-            my $try = File::Spec->catdir( @root, "bin", "jifty" );
+            my $try = File::Spec->catpath($volume, File::Spec->catdir( @root, "bin", "jifty" ), '');
             if (# XXX: Just a quick hack
                 # MSWin32's 'maybe_command' sees only file extension.
                 # Maybe we should check 'jifty.bat' instead on Win32,
@@ -163,14 +168,14 @@ sub app_root {
                 and lc($try) ne lc(File::Spec->catdir($Config::Config{bin}, "jifty"))
                 and lc($try) ne lc(File::Spec->catdir($Config::Config{scriptdir}, "jifty")) )
             {
-                return $APP_ROOT = File::Spec->catdir(@root);
+                return $APP_ROOT = File::Spec->catpath($volume, File::Spec->catdir(@root), '');
             }
             pop @root;
         }
     }
     warn "Can't guess application root from current path ("
         . Cwd::cwd()
-        . ") or bin path ($FindBin::Bin)\n";
+        . ") or bin path ($FindBin::Bin)\n" unless $args{quiet};
     return ''; # returning undef causes tons of 'uninitialized...' warnings.
 }
 
@@ -226,7 +231,13 @@ sub make_path {
     my $whole_path = shift;
     return 1 if (-d $whole_path);
     Jifty::Util->require('File::Path');
-    File::Path::mkpath([$whole_path]);
+
+    local $@;
+    eval { File::Path::mkpath([$whole_path]) };
+
+    if ($@) {
+        Jifty->log->fatal("Unable to make path: $whole_path: $@")
+    }
 }
 
 =head2 require PATH
@@ -256,16 +267,21 @@ sub _require {
 
     return 1 if $self->already_required($class);
 
-    my $pkg = $class .".pm";
-    $pkg =~ s/::/\//g;
-    my $retval = eval  {CORE::require "$pkg"} ;
+    # .pm might already be there in a weird interaction in Module::Pluggable
+    my $file = $class;
+    $file .= ".pm"
+        unless $file =~ /\.pm$/;
+
+    $file =~ s/::/\//g;
+
+    my $retval = eval  {CORE::require "$file"} ;
     my $error = $@;
     if (my $message = $error) { 
         $message =~ s/ at .*?\n$//;
-        if ($args{'quiet'} and $message =~ /^Can't locate $pkg/) {
+        if ($args{'quiet'} and $message =~ /^Can't locate $file/) {
             return 0;
         }
-        elsif ( $error !~ /^Can't locate $pkg/) {
+        elsif ( $error !~ /^Can't locate $file/) {
             die $error;
         } else {
             Jifty->log->error(sprintf("$message at %s line %d\n", (caller(1))[1,2]));
@@ -302,11 +318,10 @@ Helper function to test whether a given class has already been require'd.
 
 =cut
 
-
 sub already_required {
     my ($self, $class) = @_;
-    my $path =  join('/', split(/::/,$class)).".pm";
-    return ( $INC{$path} ? 1 : 0);
+    $class =~ s{::}{/}g;
+    return ( $INC{"$class.pm"} ? 1 : 0);
 }
 
 =head2 generate_uuid
@@ -323,6 +338,60 @@ sub generate_uuid {
     })->create_str;
 }
 
+=head2 reference_to_data Object
+
+Provides a saner output format for models than
+C<MyApp::Model::Foo=HASH(0x1800568)>.
+
+=cut
+
+sub reference_to_data {
+    my ($self, $obj) = @_;
+    (my $model = ref($obj)) =~ s/::/./g;
+    my $id = $obj->id;
+
+    # probably a file extension, from the REST rewrite
+    my $extension = $ENV{HTTP_ACCEPT} =~ /^\w+$/ ? ".$ENV{HTTP_ACCEPT}" : '';
+
+    return {
+        jifty_model_reference => 1,
+        id                    => $obj->id,
+        model                 => $model,
+        url                   => Jifty->web->url(path => "/=/model/$model/id/$id$extension"),
+    };
+}
+
+=head2 stringify LIST
+
+Takes a list of values and forces them into strings.  Right now all it does
+is concatenate them to an empty string, but future versions might be more
+magical.
+
+=cut
+
+sub stringify {
+    my $self = shift;
+
+    my @r;
+
+    for (@_) {
+        if (UNIVERSAL::isa($_, 'Jifty::Record')) {
+            push @r, Jifty::Util->reference_to_data($_);
+        }
+        if (UNIVERSAL::isa($_, 'Jifty::DateTime') && $_->is_date) {
+            push @r, $_->ymd;
+        }
+        elsif (defined $_) {
+            push @r, '' . $_; # force stringification
+        }
+        else {
+            push @r, undef;
+        }
+    }
+
+    return wantarray ? @r : $r[-1];
+}
+
 =head1 AUTHOR
 
 Various folks at Best Practical Solutions, LLC.
