@@ -281,7 +281,79 @@ sub create_columns {
     return $self->edit_columns(@_);
 }
 
+=head2 render_field mode => $mode, field => $field, action => $action
 
+Renders a particular field in a given mode (read, create, edit). This attempts
+to dispatch directly to a method with the given field name. For example, if the
+subclass has, say, an C<edit_field_post> method, then it will be preferred over
+the generic C<edit_field> method.
+
+=cut
+
+sub render_field {
+    my $self = shift;
+    my %args = @_;
+
+    my $mode  = $args{mode};
+    my $field = $args{field};
+
+    my $render_method = "${mode}_field";
+
+    $render_method = "${mode}_field_${field}"
+        if $self->can("${mode}_field_${field}");
+
+    $self->$render_method(%args);
+}
+
+=head2 view_field action => $action_object, field => $field_name
+
+Displays the column as read-only.
+
+=cut
+
+sub view_field {
+    my $self = shift;
+    my %args = @_;
+
+    render_param($args{action} => $args{field}, render_mode => 'read');
+}
+
+=head2 create_field action => $action_object, field => $field_name
+
+Displays the column for a create form.
+
+=cut
+
+sub create_field {
+    my $self = shift;
+    my %args = @_;
+
+    render_param($args{action}, $args{field});
+}
+
+=head2 edit_field action => $action_object, field => $field_name
+
+Displays the column for an edit form.
+
+=cut
+
+sub edit_field {
+    my $self = shift;
+    my %args = @_;
+
+    render_param($args{action}, $args{field});
+}
+
+=head2 page_title
+
+The title for the CRUD page
+
+=cut
+
+sub page_title {
+    my $self = shift;
+    $self->object_type;
+}
 
 =head1 TEMPLATES
 
@@ -293,7 +365,7 @@ Contains the master form and page region containing the list of items. This is m
 
 template 'index.html' => page {
     my $self = shift;
-    title is $self->object_type;
+    title is $self->page_title;
     form {
         render_region(
             name     => $self->object_type.'-list',
@@ -355,8 +427,12 @@ template 'view' => sub :CRUDView {
         my @fields = $self->display_columns($update);
         for my $field (@fields) {
             div { { class is 'view-argument-'.$field};
-            render_param( $update => $field,  render_mode => 'read'  );
-            }; 
+                $self->render_field(
+                    mode   => 'view',
+                    action => $update,
+                    field  => $field,
+                );
+            };
         }
         show ('./view_item_controls', $record, $update); 
     };
@@ -458,7 +534,7 @@ private template edit_item_controls => sub {
                 onclick => {
                     submit  => $delete,
                     confirm => _('Really delete?'),
-                    refresh => Jifty->web->current_region->parent,
+                    replace_with => '/__jifty/empty',
                 },
                 class => 'delete'
             );
@@ -683,7 +759,7 @@ Prints "No items found."
 
 =cut
 
-private template 'no_items_found' => sub {
+template 'no_items_found' => sub {
     div {
         { class is 'no_items' };
         outs( _("No items found.") );
@@ -704,8 +780,12 @@ private template 'list_items' => sub {
     my $object_type = $self->object_type;
     $collection->_do_search(); # we're going to need the results. 
     # XXX TODO, should use a real API to force the search
+
     if ( $collection->count == 0 ) {
-        show('./no_items_found');
+        render_region(
+            name => 'no_items_found',
+            path => $self->fragment_for('no_items_found'),
+        );
     }
 
     my $i = 0;
@@ -793,7 +873,11 @@ private template 'create_item' => sub {
     for my $field ($self->create_columns($action)) {
         div { 
             { class is 'create-argument-'.$field};
-            render_param($action, $field);
+            $self->render_field(
+                mode   => 'create',
+                action => $action,
+                field  => $field,
+            );
         }
     }
 };
@@ -810,7 +894,11 @@ private template 'edit_item' => sub {
     for my $field ($self->edit_columns($action)) {
         div {
             { class is 'update-argument-'.$field};
-            render_param($action, $field);
+            $self->render_field(
+                mode   => 'edit',
+                action => $action,
+                field  => $field,
+            );
         }
     }
 };
@@ -846,6 +934,7 @@ private template 'new_item_controls' => sub {
             onclick => [
                 { submit       => $create },
                 { refresh_self => 1 },
+                { delete => Jifty->web->qualified_parent_region('no_items_found') },
                 {   element => Jifty->web->current_region->parent->get_element( 'div.list'),
                     append => $self->fragment_for('view'),
                     args   => {
